#include "os.h"
#include "kheap.h"
#include "task.h"
#include "initrd.h"
#include "syscall.h"

extern ULONG file_data_start;
extern ULONG file_data_end;

extern page_directory_t* current_directory;
extern task_t* current_task;
extern task_t* ready_queue;
extern tss_entry_t tss_entry;
extern heap_t* kheap;

ULONG ramdisk_start; // In the heap
ULONG address_user  = 0x00400000; // begin of user space
UCHAR address_TEST[4096];

void f2()
{
    while(TRUE)
    {
      settextcolor(getpid(),0);
      putch(getpid()+'@');
      switch_context();
    }
}

void f3()
{
    while(TRUE)
    {
      settextcolor(getpid(),0);
      putch(getpid()+'@');
      switch_context();
    }
}

int main()
{
    k_clear_screen();
    printformat("Welcome to PrettyOS ... \n");
    // GDT, IDT, ISRS, IRQ, timer, keyboard, paging, interrupts, multitasking
    gdt_install();
    idt_install();
    isrs_install();
    irq_install();
    initODA();
    timer_install();
    keyboard_install();
    paging_install();

    // Initialise the kernel heap.
    kheap = create_heap(KHEAP_START, KHEAP_START+KHEAP_INITIAL_SIZE, KHEAP_MAX, 1, 0); // supervisor and read/write

    tasking_install();
    sti();

    settextcolor(2,0); printformat("VFS & RAM Disk install\n"); settextcolor(15,0);
    ramdisk_start = k_malloc(0x100000,0,0);
    k_memcpy((void*)ramdisk_start, &file_data_start, (ULONG)&file_data_end - (ULONG)&file_data_start); //process.asm
    fs_root = install_initrd(ramdisk_start);

    // list the content of files <- data from outside "loaded" by incbin ...
    ULONG i = 0;
    struct dirent* node = 0;
    while( (node = readdir_fs(fs_root, i)) != 0)
    {
        printformat("\nFound file %s\n",node->name);
        fs_node_t* fsnode = finddir_fs(fs_root, node->name);

        if((fsnode->flags & 0x7) == FS_DIRECTORY)
        {
            printformat("\t(directory)\n");
        }
        else
        {
            printformat("\t contents: \"");
            UCHAR buf[4096];
            ULONG sz = read_fs(fsnode, 0, fsnode->length, buf);

            ULONG j;
            for(j=0; j<sz; ++j)
            {
                if( k_strcmp(node->name,"Test-Programm")==0 )
                {
                    printformat("%d ",buf[j]);
                    address_TEST[j] = buf[j];
                }
                else
                    printformat("%c",buf[j]);
            }
            printformat("\n\n");
        }
        ++i;
    }

    k_memcpy((void*)address_user, address_TEST, 4096); // Test-Programm ==> user space

    ULONG j;
    for(j=0x400060; j<0x400100; ++j)
    {
        printformat("%d ",*((UCHAR*)j));
    }
    printformat("\n");

    sleepSeconds(10);
    settextcolor(15,0);

    k_clear_screen();
    pODA->ts_flag = 1; // enable task_switching
    create_task (f2,0);
    create_task (f3,0);
    create_task ((void*)0x400060,3);

    while(TRUE)
    {
        settextcolor(getpid(),0);
        putch(getpid()+'@');
        switch_context();
    }
    return 0;
}
